C PROGRAM RSIGMA.F  - TABARE GALLARDO, JANUARY 2006
C IMPROVED VERSION 2018
C gallardo@fisica.edu.uy
C CALCULATES THE MEAN RESONANT DISTURBING FUNCTION R(sigma).
C PLANETS ASSUMED IN CIRCULAR AND ZERO INCLINATION ORBITS
      IMPLICIT REAL*8 (A-H,J-Z)
      DIMENSION RP(400),SE(10),SM(10)
      REAL*4 RPG(400),SMA(400)
      CHARACTER*9 APE(10)
      TWOPI = 8.0D0*DATAN(1.0D0)
      CERO  = 0.0D0
      UNO   = 1.0D0
      PI=TWOPI/2.D0
      G2R=PI/180.D0
      ERROR=1.D-12
C      KGAUS=0.01720209895D0
C      KG2=KGAUS**2
C  PLANETS
      APE(1)='Mercury'
      APE(2)='Venus'
      APE(3)='Earth'
      APE(4)='Mars'
      APE(5)='Jupiter'
      APE(6)='Saturn'
      APE(7)='Uranus'
      APE(8)='Neptune'
C MEAN SEMIMAJOR AXIS
C DEFINED AS THE a THAT GENERATES THE MEAN MOTION n EQUAL TO THE OBSERVED
C dlambda/dt IN THE NUMERICAL INTEGRATIONS OF THE FULL PLANETART SYSTEM
C BEING lambda = varpi + mean anomaly
      SE(1)=  0.387099D0
      SE(2)=  0.723332D0
      SE(3)=  1.000000D0
      SE(4)=  1.523692D0
      SE(5)=  5.20281D0
      SE(6)=  9.53882D0
      SE(7)= 19.1831D0
      SE(8)= 30.0556D0
C MASSES
      SM(1)=1.66013679D-07
      SM(2)=2.44783833D-06
      SM(3)=3.04043264D-06
      SM(4)=3.22715144D-07
      SM(5)=9.54791938D-04
      SM(6)=2.85885980D-04
      SM(7)=4.36624404D-05
      SM(8)=5.15138902D-05

      WRITE(*,*)'------------------------------------------------------'
      WRITE(*,*)'THIS PROGRAM EVALUATES R(sigma) AND ITS STRENGTH SR   '
      WRITE(*,*)'SR = <R>-Rmin. (Gallardo 2006, Icarus 184, 29)'
      WRITE(*,*)'Tabare Gallardo                gallardo@fisica.edu.uy '
      WRITE(*,*)'version 2018'
      WRITE(*,*)'------------------------------------------------------'

      WRITE(*,*)'PLANET? (Mer=1,...,Nep=8)'
      READ(*,*) INPLA
      WRITE(*,*)'RESONANCE |p+q|,|p| ?'
      WRITE(*,*)'(ex. 2,3 for plutinos; 3,2 for hildas)'
      READ(*,*) NPQ,NP
C NPQ=|P+Q|
C NP=|P|
C ORDER q
      ORQ=DABS(NPQ-NP)
      MAXFAC=NPQ
      IF(NPQ.GT.NP) THEN
C INTERIOR RESONANCE
        WRITE(*,*)'sigma =',int(NPQ),'*lambda_pla',int(-np),'*lambda',
     *int(-ORQ),'*varpi'
      ELSE
C      IF(NPQ.LE.NP) THEN
C IT IS AN EXTERIOR RESONANCE  OR TROJANS
        MAXFAC=NP
        NPQ=-NPQ
        NP=-NP
      WRITE(*,*)'sigma =',int(NPQ),'*lambda_pla +',int(-np),'*lambda -',
     *int(ORQ),'*varpi'
      ENDIF

      WRITE(*,*) '   '
 111  WRITE(*,*)'ASTEROID''S ORBITAL ELEMENTS:  e,i,argper'
C sigma is invariant to changes in aries then I take aries at the node
C so longnode(particle)=0
C The planet is in circular and i=0 orbit  then the longper is not defined
C so I take \omega=\Omega=0 for the planet
      READ(*,*)EXC,YNC,argper
      LONOD=0.D0
      LOPER=argper
C THE PLANET IS 1
      A1=SE(INPLA)
      E1=0.D0
      J1G=0.D0
      L1G=0.D0
      P1G=0.D0
      J1=J1G*G2R
      L1=L1G*G2R
      P1=P1G*G2R
C PARTICLE IS 2
      A2=A1*((DABS(NP/NPQ))**2/(1.d0+sm(inpla)))**(1.D0/3.D0)
      SALSEM=A2
      E2=EXC
      J2GRA=YNC
      J2=J2GRA*G2R
      L2G=LONOD
      P2G=LOPER
      L2=L2G*G2R
      P2=P2G*G2R
C NUMBER OF EVALUATIONS OF R(sigma) BETWEEN O AND 360
      ISIMAX=180
C STEPS OF THE NUMERICAL INTEGRATION FTOM 0 TO 2PI*p IN lambda_pla
      IPASOS=1000*INT(MAXFAC)
C sigma = (p+q)*lambda_p - p*lambda - q*varpi
      DO ISI=1,ISIMAX
C acrit is sigma
        ACRIT=DFLOAT(ISI-1)/DFLOAT(ISIMAX)*360.D0
        TETA=ACRIT+ORQ*LOPER
 901    IF (TETA.GT.360.D0) THEN
          TETA=TETA-360.D0
          GOTO 901
        END IF
C to radians
        TETAR=TETA*G2R
        RTOT=CERO
      DO ILAMBDA1=1,IPASOS
C all possible configurations occur after p revolutions of the planet
C LAMBDA PLANET
      LA1=DFLOAT(ILAMBDA1)/DFLOAT(IPASOS)*TWOPI*DABS(NP)
C MEAN ANOMALY PLANET
      AM1=LA1-P1
 200  IF (AM1.GT.TWOPI) THEN
        AM1=AM1-TWOPI
        GOTO 200
      END IF
 201  IF (AM1.LT.CERO) THEN
        AM1=AM1+TWOPI
        GOTO 201
      END IF
C GIVEN LAMBDA PLANET CALCULATE LAMBDA PARTICLE
      LA2=(NPQ*LA1-TETAR)/NP
C MEAN ANOMALY PARTICLE
      AM2=LA2-P2
 202  IF (AM2.GT.TWOPI) THEN
        AM2=AM2-TWOPI
        GOTO 202
      END IF
 203  IF (AM2.LT.CERO) THEN
        AM2=AM2+TWOPI
        GOTO 203
      END IF
C CRUDE METHOD FOR SOLVING KEPLER FOR PLANET
C JUST IN CASE YOU WANTED TO PUT SOME PLANETARY ECCENTRICITY
      AEX=AM1
 210     AEXN=AM1+E1*DSIN(AEX)
      IF(DABS(AEXN-AEX).GT.ERROR) THEN
        AEX=AEXN
        GOTO 210
      ENDIF
      AEX1=AEXN
C TRUE ANOMALY
      AVE1=DACOS((DCOS(AEX1)-E1)/(UNO-E1*DCOS(AEX1)))
      IF(AM1.GT.PI) AVE1=TWOPI-AVE1
C HELIOCENTRIC DISTANCE
      R1=A1*(UNO-E1*DCOS(AEX1))
C CRUDE METHOD FOR SOLVING KEPLER FOR PARTICLE (ECCENTRIC)
C      AEX=AM2
C 211     AEXN=AM2+E2*DSIN(AEX)
C      IF(DABS(AEXN-AEX).GT.ERROR) THEN
C        AEX=AEXN
C        GOTO 211
C      ENDIF
C      AEX2=AEXN
C SOLVING KEPLER FOR PARTICLE BY CLEVER METHOD
      CALL SOLKEP(E2,AM2,AEX2,INITER)
C TRUE ANOMALY
      AVE2=DACOS((DCOS(AEX2)-E2)/(UNO-E2*DCOS(AEX2)))
      IF(AM2.GT.PI) AVE2=TWOPI-AVE2
C HELIOCENTRIC DISTANCE
      R2=A2*(UNO-E2*DCOS(AEX2))
C RADIUS VECTOR FOR PLANET
      X1=R1*(DCOS(L1)*DCOS(P1-L1+AVE1)-DSIN(L1)*DSIN(P1-L1+AVE1)
     **DCOS(J1))
      Y1=R1*(DSIN(L1)*DCOS(P1-L1+AVE1)+DCOS(L1)*DSIN(P1-L1+AVE1)
     **DCOS(J1))
      Z1=R1*DSIN(P1-L1+AVE1)*DSIN(J1)
C RADIUS VECTOR FOR PARTICLE
      X2=R2*(DCOS(L2)*DCOS(P2-L2+AVE2)-DSIN(L2)*DSIN(P2-L2+AVE2)
     **DCOS(J2))
      Y2=R2*(DSIN(L2)*DCOS(P2-L2+AVE2)+DCOS(L2)*DSIN(P2-L2+AVE2)
     **DCOS(J2))
      Z2=R2*DSIN(P2-L2+AVE2)*DSIN(J2)
C DISTANCE
      DELTA=DSQRT((X1-X2)**2+(Y1-Y2)**2+(Z1-Z2)**2)
C SCALAR PRODUCT R1*R2
      R1R2=X1*X2+Y1*Y2+Z1*Z2
C DIRECT PART
      RDIR=UNO/DELTA
C INDIRECT PART (PROBABLY THIS IS A WASTE OF TIME)
      RIND=-R1R2/R1**3
C DISTURBING FUNCTION
      RPER=RDIR+RIND
C SUMATION FOR A CRUDE INTEGRAL
      RTOT=RTOT+RPER
      ENDDO
      RTOT=RTOT/DFLOAT(IPASOS)
C NOW MULTIPLY BY MASS_PLANET/MASS_JUPITER
      RP(ISI)=RTOT*SM(INPLA)/SM(5)
C TO OBTAIN STANDARD UNITS MULTIPLY RP BY MASS_JUPITER*KGAUSS**2
      SMA(ISI)=ACRIT
      ENDDO
C CALCULATION OF MEAN VALUE <R>
      VMEDIO=0.D0
      DO I2=1,ISIMAX
         VMEDIO=VMEDIO+RP(I2)
      ENDDO
      VMEDIO=VMEDIO/DFLOAT(ISIMAX)
C FIND R_max AND R_min
      VRMAX=-999999.D0
      VRMIN=999999.D0
      DO I2=1,ISIMAX
        IF(RP(I2).LT.VRMIN) THEN
          VRMIN=RP(I2)
        ENDIF
        IF(RP(I2).GT.VRMAX) THEN
          VRMAX=RP(I2)
        ENDIF
      ENDDO
      CLOSE(1)




      OPEN(2,FILE="strength.dat",STATUS="UNKNOWN",ACCESS="APPEND")
      WRITE(2,*)'Planet =  ',APE(INPLA)
      WRITE(2,*)'Resonance =  ',DABS(NPQ),':',DABS(NP)
      WRITE(2,*)'a = ',SALSEM
      WRITE(2,*)'e = ',EXC
      WRITE(2,*)'i = ',YNC
      WRITE(2,*)'arg perihelion = ',ARGPER
      WRITE(2,*)'<R> = ',VMEDIO
      WRITE(2,*)'Strength (<R>-Rmin) = ',(VMEDIO-VRMIN)
      WRITE(2,*)'(Units: G*M_jup = 1)'
      WRITE(2,*)'        '
      CLOSE(2)
      WRITE(*,*)'        '
      WRITE(*,*)'Planet =  ',APE(INPLA)
      WRITE(*,*)'Resonance =  ',DABS(NPQ),':',DABS(NP)
      WRITE(*,*)'a = ',SALSEM
      WRITE(*,*)'e = ',EXC
      WRITE(*,*)'i = ',YNC
      WRITE(*,*)'arg perihelion = ',ARGPER
      WRITE(*,*)'<R> = ',VMEDIO
      WRITE(*,*)'Strength (<R>-Rmin) = ',(VMEDIO-VRMIN)
      WRITE(*,*)'(Units: G*M_jup = 1)'
      WRITE(*,*)'        '


C CALCULATION OF R - R_min FOR PLOTTING WITH DISLIN LIBRARIES
       DO I=1,ISIMAX
         RPG(I)=(RP(I)-VRMIN)/(VRMAX-VRMIN)
       ENDDO
      OPEN(1,FILE="rsigma.dat",STATUS="UNKNOWN")
      WRITE(1,*)'  sigma      R(sigma)      '
      do isi=1,isimax
      WRITE(1,1000) sma(isi),rpg(isi)
      ENDDO
C AT THE END REPEAT RESULT FOR SIGMA=0
      WRITE(1,1000) 360.0,RPg(1)
      close(1)



C  DISLIN  START HERE

C IF YOU DONT LIKE TO COMPILE WITH DISLIN CUT HERE ---------------------
C      call METAFL('XWIN')
C      CALL DISINI
C      CALL PAGHDR('Resonant Disturbing Function','T. Gallardo',4,0)
C      CALL COMPLX
C      CALL NAME('sigma','X')
C      CALL TITLIN('Shape of the Resonant Disturbing Function',3)
C      CALL GRAF(0.,360.,0.,60.,0.,1.,0.,0.2)
C      CALL QPLOT(SMA,RPG,ISIMAX)
C      CALL ENDGRF
C      CALL DISFIN
C ----------------END DISLIN-------------------------------------------

      GOTO 111
 1000 FORMAT(F7.2,E16.8)
      END

C ========================================================================
C SOLVING KEPLER
      SUBROUTINE SOLKEP(EX,M,E,NITER)
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 M,MK
      TOLE=1.D-12
      TWOPI=8.D0*DATAN(1.D0)
      M=DMOD(M,TWOPI)
      E=M
      NITER=0
 100  E0=E
      SE=DSIN(E0)
      CE=DCOS(E0)
      ES=EX*SE
      EC=1.D0-EX*CE
      MK=E0-ES
      U=(MK-M)/EC
      XPRI=E0-U
      XSEG=E0-U/(1.D0-U*ES)
      E=(XPRI+XSEG)/2.D0
      DEX=DABS(E-E0)
      NITER=NITER+1
C	 IF(NITER.GT.20)GOTO 200
      IF(DEX.GT.TOLE)GOTO 100
      RETURN
      END
C ========================================================================





